@@ -913,13 +913,17 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
       return;
     }
 
-  if (priv->layout && HILDON_DESKTOP_IS_HOME_ITEM (applet))
+  if (HILDON_DESKTOP_IS_HOME_ITEM (applet))
     {
-      GdkRectangle *rect;
-      const gchar *name = hildon_desktop_item_get_id (
-                                     HILDON_DESKTOP_ITEM (applet));
+      GdkRectangle *rect = NULL;
 
-      rect = g_hash_table_lookup (priv->layout, name);
+      if (priv->layout)
+        {
+          const gchar *name =
+              hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (applet));
+
+          rect = g_hash_table_lookup (priv->layout, name);
+        }
 
       if (rect)
         {
@@ -929,7 +933,6 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
           if (rect->width > 0 && rect->height > 0)
             gtk_widget_set_size_request (applet, rect->width, rect->height);
         }
-
       else
         {
           hildon_home_area_put (HILDON_HOME_AREA (area), applet, 0, 0);
@@ -941,8 +944,6 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
         hildon_desktop_home_item_set_layout_mode (HILDON_DESKTOP_HOME_ITEM (applet),
                                             TRUE);
     }
-
-
 }
 
 static void
@@ -1326,7 +1327,7 @@ hildon_home_area_expose (GtkWidget *widget,
       XserverRegion             region;
       GdkDrawable              *drawable;
       Window                   *wchildren, root, parent;
-      int                       n_children, i;
+      guint                     n_children, i;
 
       gdk_window_get_internal_paint_info (widget->window,
                                           &drawable,
