@@ -443,6 +443,19 @@ hd_wm_watchable_app_get_exec (HDWMWatchableApp *app)
   return app->exec_name;
 }
 
+void 
+hd_wm_watchable_app_dummy_set_name (HDWMWatchableApp *app, const gchar *name)
+{
+  if (!hd_wm_watchable_app_is_dummy (app))
+    return;
+
+  g_free (app->app_name);
+  g_free (app->class_name);
+    
+  app->app_name   = g_strdup (name);
+  app->class_name = g_strdup (name);
+}	
+
 const gchar*
 hd_wm_watchable_app_get_name (HDWMWatchableApp *app)
 {
