@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <sys/resource.h>
 #include <string.h>
+#include <fcntl.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -279,7 +280,10 @@ hd_applications_menu_key_press (GtkWidget *menu,
     
     return TRUE;
   }
-  else if (event->keyval == GDK_Right ||
+  else if (event->keyval == GDK_Return    ||
+	   event->keyval == GDK_KP_Enter  ||
+	   event->keyval == GDK_ISO_Enter ||
+	   event->keyval == GDK_Right     ||
            event->keyval == GDK_KP_Right)
   {
     if (hd_applications_menu_has_focus (button->priv->menu_categories))
@@ -328,6 +332,18 @@ hd_applications_menu_key_press (GtkWidget *menu,
   return FALSE;
 }
 
+static gboolean 
+hd_applications_menu_button_release_category (GtkMenuItem *item, 
+					    GdkEventButton *event,
+					    HDApplicationsMenu *button)
+{
+  button->priv->focus_applications = TRUE;
+
+  gtk_menu_item_activate (item);
+
+  return TRUE;
+}
+
 static void
 hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *button)
 {
@@ -470,6 +486,8 @@ hd_applications_menu_activate_app (GtkMenuItem *item, HDApplicationsMenu *button
       {
         int priority;
         errno = 0;
+        gchar *oom_filename;
+        int fd;
 
         /* If the child process inherited desktop's high priority,
          * give child default priority */
@@ -479,6 +497,19 @@ hd_applications_menu_activate_app (GtkMenuItem *item, HDApplicationsMenu *button
         {
           setpriority (PRIO_PROCESS, child_pid, 0);
         }
+
+        /* Unprotect from OOM */
+        oom_filename = g_strdup_printf ("/proc/%i/oom_adj",
+                                        child_pid);
+        fd = open (oom_filename, O_WRONLY);
+        g_free (oom_filename);
+
+        if (fd >= 0)
+          {
+            write (fd, "0", sizeof (char));
+            close (fd);
+          }
+
       }
     }
     else
@@ -598,6 +629,11 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       		              sub_items, 
 			      (GDestroyNotify) hd_applications_menu_free_menu_items);
 
+      g_signal_connect (G_OBJECT (menu_item), 
+		        "button-release-event",
+      		        G_CALLBACK (hd_applications_menu_button_release_category),
+      		        button);
+
       g_signal_connect (G_OBJECT (menu_item), 
 		        "activate",
       		        G_CALLBACK (hd_applications_menu_activate_category),
@@ -737,7 +773,7 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   
   popup_window =
     HILDON_DESKTOP_POPUP_WINDOW 
-      (hildon_desktop_popup_window_new (1, 
+      (hildon_desktop_popup_window_new (0, 
 					GTK_ORIENTATION_HORIZONTAL,
 					HD_POPUP_WINDOW_DIRECTION_RIGHT_BOTTOM));
 
@@ -1045,23 +1081,19 @@ hd_applications_menu_register_monitors (HDApplicationsMenu * button)
   gchar	*file = NULL;
   gchar *conf_file;
 
-  conf_file = g_build_filename (g_get_home_dir (), USER_MENU_FILE, NULL);
- 
+  conf_file = g_build_filename (g_get_home_dir (), USER_MENU_DIR, MENU_FILE, NULL);
+
   /* We copy SYSTEMWIDE_MENU_FILE to always track the changes */
   if (!g_file_test (conf_file, G_FILE_TEST_EXISTS))
-    if (g_file_get_contents (SYSTEMWIDE_MENU_FILE, &file, NULL, NULL))
+    if (g_file_get_contents (HD_DESKTOP_MENU_PATH "/" MENU_FILE, &file, NULL, NULL))
     {
-      g_debug ("I couldn't get contents");
-      g_file_set_contents (USER_MENU_FILE, file, -1, NULL);
+      g_file_set_contents (conf_file, file, -1, NULL);
     }
   
   g_free (file);
   
-  /* Watch systemwide menu conf */
-  dir = g_path_get_dirname (SYSTEMWIDE_MENU_FILE);
- 
   if (gnome_vfs_monitor_add (&button->priv->system_dir_monitor, 
-                             dir,
+                             HD_DESKTOP_MENU_PATH,
                              GNOME_VFS_MONITOR_DIRECTORY,
                              (GnomeVFSMonitorCallback) hd_applications_menu_dir_changed,
                              button) != GNOME_VFS_OK)
@@ -1070,39 +1102,33 @@ hd_applications_menu_register_monitors (HDApplicationsMenu * button)
       	       "failed setting monitor callback "
       	       "for systemwide menu conf." );
   }
-
-  g_free (dir);
   
-  /* Watch user specific menu conf */
-  if (g_get_home_dir ())
-  {    
-    /* Have to get the directory from the path because the USER_MENU_FILE
-       define might contain directory (it does, in fact). */
-    dir = g_path_get_dirname (conf_file);
-    
-    g_mkdir (dir, 0755);
+  /* Have to get the directory from the path because the USER_MENU_FILE
+     define might contain directory (it does, in fact). */
+  dir = g_path_get_dirname (conf_file);
+  
+  g_mkdir (dir, 0755);
 
-    if (dir && *dir)
-    {
-      if (gnome_vfs_monitor_add (&button->priv->home_dir_monitor, 
-                                 dir,
-                                 GNOME_VFS_MONITOR_DIRECTORY,
-                                 (GnomeVFSMonitorCallback) hd_applications_menu_dir_changed,
-                                 button) != GNOME_VFS_OK)
-      {
-        g_warning ("Others_menu_initialize_menu: "
-      	           "failed setting monitor callback "
-      	           "for user specific menu conf." );
-      }
-    }
-    else
+  if (dir && *dir)
+  {
+    if (gnome_vfs_monitor_add (&button->priv->home_dir_monitor, 
+                               dir,
+                               GNOME_VFS_MONITOR_DIRECTORY,
+                               (GnomeVFSMonitorCallback) hd_applications_menu_dir_changed,
+                               button) != GNOME_VFS_OK)
     {
       g_warning ("Others_menu_initialize_menu: "
-    	         "failed to create directory '%s'", dir);
+    	         "failed setting monitor callback "
+    	         "for user specific menu conf." );
     }
-
-    g_free (dir);
   }
+  else
+  {
+    g_warning ("Others_menu_initialize_menu: "
+  	       "failed to create directory '%s'", dir);
+  }
+
+  g_free (dir);
       
   g_free (conf_file);
   
@@ -1118,6 +1144,19 @@ hd_applications_menu_register_monitors (HDApplicationsMenu * button)
       	       "failed setting monitor callback "
       	       "for .desktop directory." );
   }
+
+  /* Monitor the hildon .desktop directories, so we can regenerate the 
+   * menu when a new application is installed */
+  if (gnome_vfs_monitor_add (&button->priv->desktop_dir_monitor, 
+                             HD_DESKTOP_ENTRY_PATH "/hildon",
+                             GNOME_VFS_MONITOR_DIRECTORY,
+                             (GnomeVFSMonitorCallback) hd_applications_menu_dir_changed,
+                             button) != GNOME_VFS_OK)
+  {
+    g_warning ("Others_menu_initialize_menu: "
+      	       "failed setting monitor callback "
+      	       "for .desktop directory." );
+  }
 }
 
 void
