@@ -83,6 +83,9 @@ void        hd_home_background_apply_async      (HDHomeBackground *background,
 HDHomeBackground *
             hd_home_background_copy             (const HDHomeBackground *src);
 
+gboolean    hd_home_background_equal            (const HDHomeBackground *bg1,
+                                                 const HDHomeBackground *bg2);
+
 G_END_DECLS
 
 #endif /* __HD_HOME_BACKGROUND_H__ */
