@@ -43,6 +43,8 @@
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkspinbutton.h>
 
+#include <X11/Xatom.h>
+
 #ifdef HAVE_LIBHILDON
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-note.h>
@@ -110,10 +112,16 @@ struct _HDHomeWindowPrivate
   HDHomeBackground *background;
   HDHomeBackground *previous_background;
 
-  gboolean          is_inactive;
+  gboolean          screen_is_off;
 
   gboolean          selecting_applets;
   guint             save_area_timeout;
+
+  /* Cancelling the background loading */
+  GtkWidget        *cancel_note;
+  guint             background_loading_timeout;
+  guint             cancel_note_update_timeout;
+  gboolean          background_cancelled;
 };
 
 /* Properties */
@@ -244,8 +252,8 @@ hd_home_window_background (HDHomeWindow        *window,
                            gboolean             background);
 
 static void
-hd_home_window_system_inactivity (HDHomeWindow     *window,
-                                  gboolean              is_inactive);
+hd_home_window_screen_off (HDHomeWindow     *window,
+                           gboolean          is_off);
 
 static void
 hd_home_window_area_add (HDHomeWindow *window);
@@ -274,6 +282,12 @@ hd_home_window_button1_clicked (HildonHomeWindow *window);
 static void
 hd_home_window_button2_clicked (HildonHomeWindow *window);
 
+static void
+hd_home_window_cancel_note_hide (HDHomeWindow *window);
+
+static void
+hd_home_window_cancel_note_show (HDHomeWindow *window);
+
 #if 0
 static void
 hd_home_window_adjust_alpha_activate (HDHomeWindow *window);
@@ -308,7 +322,7 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   window_class->layout_mode_accept = hd_home_window_layout_mode_accept;
   window_class->layout_mode_cancel = hd_home_window_layout_mode_cancel;
   window_class->background         = hd_home_window_background;
-  window_class->system_inactivity  = hd_home_window_system_inactivity;
+  window_class->screen_off         = hd_home_window_screen_off;
 
   pspec = g_param_spec_pointer ("osso-context",
                                 "Osso Context",
@@ -347,10 +361,10 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
                 1,
                 G_TYPE_BOOLEAN);
 
-  g_signal_new ("system-inactivity",
+  g_signal_new ("screen-off",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_FIRST,
-                G_STRUCT_OFFSET (HDHomeWindowClass, system_inactivity),
+                G_STRUCT_OFFSET (HDHomeWindowClass, screen_off),
                 NULL,
                 NULL,
                 g_cclosure_marshal_VOID__BOOLEAN,
@@ -638,6 +652,23 @@ background_apply_callback (HDHomeBackground *background,
   HDHomeWindowPrivate  *priv;
   g_debug ("Background applied!");
 
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+
+  if (priv->background_loading_timeout > 0)
+    {
+      g_source_remove (priv->background_loading_timeout);
+      priv->background_loading_timeout = 0;
+    }
+
+  if (priv->background_cancelled)
+    {
+      priv->background_cancelled = FALSE;
+      return;
+    }
+
+  if (GTK_IS_WIDGET (priv->cancel_note))
+    hd_home_window_cancel_note_hide (window);
+
   if (error)
     {
       g_warning ("Got error when apply background: %s",
@@ -648,8 +679,6 @@ background_apply_callback (HDHomeBackground *background,
   gdk_window_clear (GTK_WIDGET (window)->window);
   gtk_widget_queue_draw (GTK_WIDGET (window));
 
-  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-
   if (background != priv->background)
     {
       if (priv->background)
@@ -668,11 +697,16 @@ background_apply_and_save_callback (HDHomeBackground *background,
   HDHomeWindowPrivate  *priv;
   gchar                *conffile;
   GError               *save_error = NULL;
+  gboolean              background_cancelled;
+
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+
+  background_cancelled = priv->background_cancelled;
 
   background_apply_callback (background, pixmap_xid, error, window);
 
   /* Do not save if an error occurred */
-  if (error)
+  if (error || background_cancelled)
     return;
 
   conffile = g_build_path (G_DIR_SEPARATOR_S,
@@ -681,8 +715,6 @@ background_apply_and_save_callback (HDHomeBackground *background,
                            HD_HOME_BACKGROUND_CONF_FILE,
                            NULL);
 
-  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-
   hd_home_background_save (priv->background,
                            conffile,
                            &save_error);
@@ -699,10 +731,92 @@ background_apply_and_save_callback (HDHomeBackground *background,
 
 }
 
+static GdkFilterReturn
+hd_home_window_root_event_filter (GdkXEvent *xevent,
+                                  GdkEvent  *event,
+                                  gpointer   user_data)
+{
+  HDHomeWindow         *window = user_data;
+  static Atom           active_window_atom = 0;
+  static gboolean       home_is_topmost = TRUE;
+  XPropertyEvent       *prop = NULL;
+
+  if (((XEvent *) xevent)->type != PropertyNotify)
+    return GDK_FILTER_CONTINUE;
+
+  if (active_window_atom == 0)
+    {
+      active_window_atom = XInternAtom (GDK_DISPLAY (),
+                                        "_MB_CURRENT_APP_WINDOW",
+                                        False);
+
+    }
+
+  prop = (XPropertyEvent *) xevent;
+  if ((prop->window == GDK_ROOT_WINDOW ()) &&
+      (prop->atom == active_window_atom))
+    {
+      Atom real_type;
+      gint error_trap = 0;
+      int format, status;
+      unsigned long n_res, extra;
+      Window my_window;
+      union {
+        Window *win;
+        unsigned char *pointer;
+      } res;
+
+      gdk_error_trap_push ();
+
+      status = XGetWindowProperty (GDK_DISPLAY (),
+                                   GDK_ROOT_WINDOW (),
+                                   active_window_atom,
+                                   0L, G_MAXLONG,
+                                   False, XA_WINDOW, &real_type,
+                                   &format, &n_res,
+                                   &extra, (unsigned char **) &res.pointer);
+
+      gdk_error_trap_pop ();
+
+      if ((status != Success) || (real_type != XA_WINDOW) ||
+          (format != 32) || (n_res != 1) || (res.win == NULL) ||
+          (error_trap != 0))
+        {
+          return GDK_FILTER_CONTINUE;
+        }
+
+      my_window = GDK_WINDOW_XID (GTK_WIDGET (window)->window);
+
+      if ((res.win[0] != my_window) &&
+          (home_is_topmost == TRUE))
+        {
+          home_is_topmost = FALSE;
+          g_signal_emit_by_name (G_OBJECT (window),
+                                 "background",
+                                 !home_is_topmost);
+        }
+      else if ((res.win[0] == my_window) && (home_is_topmost == FALSE))
+        {
+          home_is_topmost = TRUE;
+          g_signal_emit_by_name (G_OBJECT (window),
+                                 "background",
+                                 !home_is_topmost);
+        }
+
+
+      if (res.win)
+        XFree (res.win);
+
+    }
+
+  return GDK_FILTER_CONTINUE;
+}
+
 static void
 hd_home_window_realize (GtkWidget *widget)
 {
-  HDWM *wm;
+  HDWM         *wm;
+  GdkWindow    *root_window;
 
   GTK_WIDGET_CLASS (hd_home_window_parent_class)->realize (widget);
 
@@ -711,6 +825,12 @@ hd_home_window_realize (GtkWidget *widget)
                 "desktop-window", (gint)GDK_WINDOW_XID (widget->window),
                 NULL);
 
+  root_window = gdk_get_default_root_window ();
+  gdk_window_set_events (root_window,
+                         gdk_window_get_events (root_window)
+                         | GDK_PROPERTY_CHANGE_MASK);
+  gdk_window_add_filter (root_window, hd_home_window_root_event_filter, widget);
+
 }
 
 static void
@@ -747,101 +867,9 @@ hd_home_window_map_event (GtkWidget    *widget,
                                                                     event);
 }
 
-static const gchar *
-hd_home_window_get_pixmap_name (HDHomeWindow *window,
-                                const gchar *key)
-{
-  GtkStyle *style;
-  GtkSettings *settings;
-
-  settings = gtk_widget_get_settings (GTK_WIDGET (window));
-
-  style = gtk_rc_get_style_by_paths (settings,
-                                     key,
-                                     NULL,
-                                     G_TYPE_NONE);
-
-  if (style && style->rc_style->bg_pixmap_name[0])
-    return style->rc_style->bg_pixmap_name[0];
-
-  return NULL;
-
-}
-
 static void
 hd_home_window_style_set (GtkWidget *widget, GtkStyle *old_style)
 {
-  HDHomeWindow         *window;
-  HDHomeWindowPrivate  *priv;
-  const gchar          *north_border;
-  const gchar          *west_border;
-
-  window = HD_HOME_WINDOW (widget);
-  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-
-  north_border = hd_home_window_get_pixmap_name (window,
-                                                 HD_HOME_WINDOW_STYLE_NORTH_BORDER);
-
-  west_border  = hd_home_window_get_pixmap_name (window,
-                                                 HD_HOME_WINDOW_STYLE_WEST_BORDER);
-
-  g_debug ("style set with %s, %s", north_border, west_border);
-
-
-  /* avoid resetting the background when the window is exposed for the
-   * first time
-   */
-  if (!old_style ||
-      (north_border && !g_str_equal (priv->north_border, north_border)) ||
-      (west_border  && !g_str_equal (priv->west_border, west_border)))
-    {
-      if (north_border)
-        {
-          g_free (priv->north_border);
-          priv->north_border = g_strdup (north_border);
-        }
-      if (west_border)
-        {
-          g_free (priv->west_border);
-          priv->west_border  = g_strdup (west_border);
-        }
-
-      if (priv->background)
-        {
-          GValue value = {0};
-          g_value_init (&value, G_TYPE_STRING);
-
-          if (priv->north_border)
-            {
-              g_value_set_string (&value, priv->north_border);
-              g_object_set_property (G_OBJECT (priv->background),
-                                     "north-border",
-                                     &value);
-            }
-
-          if (priv->west_border)
-            {
-              g_value_set_string (&value, priv->west_border);
-              g_object_set_property (G_OBJECT (priv->background),
-                                     "west-border",
-                                     &value);
-            }
-
-          if (GTK_WIDGET_MAPPED (widget))
-            {
-              GdkRectangle *workarea;
-              g_object_get (widget, "work-area", &workarea, NULL);
-
-              hd_home_background_apply_async (priv->background,
-                                              widget->window,
-                                              workarea,
-                                              (HDHomeBackgroundApplyCallback)
-                                              background_apply_callback,
-                                              widget);
-            }
-        }
-    }
-
 }
 
 static gboolean
@@ -1423,6 +1451,31 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
 }
 
 #ifdef HAVE_LIBOSSO
+
+static void
+hd_home_window_osso_lowmem_cb (osso_hw_state_t *state,
+                               gpointer         window)
+{
+  g_return_if_fail (state);
+
+  g_signal_emit_by_name (G_OBJECT (window),
+                         "lowmem",
+                         state->memory_low_ind);
+
+}
+
+static void
+hd_home_window_osso_display_state_cb (osso_display_state_t      state,
+                                      gpointer                  window)
+{
+  g_return_if_fail (state);
+
+  g_signal_emit_by_name (G_OBJECT (window),
+                         "screen-off",
+                         state == OSSO_DISPLAY_OFF);
+
+}
+
 static void
 hd_home_window_set_osso_context (HDHomeWindow *window,
                                  osso_context_t *osso_context)
@@ -1435,9 +1488,20 @@ hd_home_window_set_osso_context (HDHomeWindow *window,
 
   if (priv->osso_context != osso_context)
   {
+    osso_hw_state_t hs = { 0 };
+
     priv->osso_context = osso_context;
     g_object_notify (G_OBJECT (window), "osso-context");
 
+    osso_hw_set_display_event_cb (osso_context,
+                                  hd_home_window_osso_display_state_cb,
+                                  window);
+
+    hs.memory_low_ind = TRUE;
+    osso_hw_set_event_cb (osso_context,
+                          &hs,
+                          hd_home_window_osso_lowmem_cb,
+                          window);
   }
 }
 #endif
@@ -1564,7 +1628,7 @@ hd_home_window_background (HDHomeWindow    *window,
   area = GTK_BIN (window)->child;
   g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
-  if (!priv->is_inactive)
+  if (!priv->screen_is_off)
     {
 
       /* If we were in layout mode and went to background, we need
@@ -1586,8 +1650,8 @@ hd_home_window_background (HDHomeWindow    *window,
 }
 
 static void
-hd_home_window_system_inactivity (HDHomeWindow         *window,
-                                  gboolean              is_inactive)
+hd_home_window_screen_off (HDHomeWindow         *window,
+                           gboolean              is_off)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
   GtkWidget            *area;
@@ -1595,11 +1659,11 @@ hd_home_window_system_inactivity (HDHomeWindow         *window,
   area = GTK_BIN (window)->child;
   g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
-  priv->is_inactive = is_inactive;
+  priv->screen_is_off = is_off;
 
   gtk_container_foreach (GTK_CONTAINER (area),
                          (GtkCallback)hildon_desktop_home_item_set_is_background,
-                         (gpointer)is_inactive);
+                         (gpointer)is_off);
 
 }
 
@@ -1669,6 +1733,17 @@ hd_home_window_accept_layout (HDHomeWindow *window)
   g_signal_emit_by_name (window, "layout-mode-accept");
 }
 
+static gboolean
+hd_home_window_set_background_timeout (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+
+  priv->background_loading_timeout = 0;
+
+  hd_home_window_cancel_note_show (window);
+  return FALSE;
+}
+
 static void
 hd_home_window_set_background_reponse (HDHomeWindow *window,
                                        gint response,
@@ -1687,36 +1762,56 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
     {
       case GTK_RESPONSE_OK:
           gtk_widget_hide (GTK_WIDGET(dialog));
-          g_debug ("BACKGROUND_OK");
           if (!hd_home_background_equal (priv->background,
                                          background))
-            hd_home_background_apply_async 
-                (background,
-                 GTK_WIDGET (window)->window,
-                 workarea,
-                 (HDHomeBackgroundApplyCallback)background_apply_and_save_callback,
-               window);
+            {
+              priv->background_loading_timeout =
+                  g_timeout_add (1000,
+                                 (GSourceFunc)
+                                   hd_home_window_set_background_timeout,
+                                 window);
+              hd_home_background_apply_async
+                  (hd_home_background_copy (background),
+                   GTK_WIDGET (window)->window,
+                   workarea,
+                   (HDHomeBackgroundApplyCallback)background_apply_and_save_callback,
+                   window);
+            }
           g_object_unref (priv->previous_background);
           break;
       case HILDON_HOME_SET_BG_RESPONSE_PREVIEW:
-          hd_home_background_apply_async 
-              (background,
-               GTK_WIDGET (window)->window,
-               workarea,
-               (HDHomeBackgroundApplyCallback)background_apply_callback,
-               window);
+            {
+              priv->background_loading_timeout =
+                  g_timeout_add (1000,
+                                 (GSourceFunc)
+                                   hd_home_window_set_background_timeout,
+                                 window);
+              hd_home_background_apply_async
+                  (hd_home_background_copy (background),
+                   GTK_WIDGET (window)->window,
+                   workarea,
+                   (HDHomeBackgroundApplyCallback)background_apply_callback,
+                   window);
+            }
 
           break;
       case GTK_RESPONSE_CANCEL:
       case GTK_RESPONSE_DELETE_EVENT:
           if (!hd_home_background_equal (priv->background,
                                          priv->previous_background))
-            hd_home_background_apply_async 
-                (priv->previous_background,
-                 GTK_WIDGET (window)->window,
-                 workarea,
-                 (HDHomeBackgroundApplyCallback)background_apply_callback,
-                 window);
+            {
+              priv->background_loading_timeout =
+                  g_timeout_add (1000,
+                                 (GSourceFunc)
+                                   hd_home_window_set_background_timeout,
+                                 window);
+              hd_home_background_apply_async
+                  (priv->previous_background,
+                   GTK_WIDGET (window)->window,
+                   workarea,
+                   (HDHomeBackgroundApplyCallback)background_apply_callback,
+                   window);
+            }
 
           gtk_widget_hide (GTK_WIDGET (dialog));
           g_object_unref (priv->previous_background);
@@ -1724,12 +1819,14 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
       default:
           break;
     }
+
+  g_object_unref (background);
 }
 
 static void
 hd_home_window_layout_mode_activate (HDHomeWindow *window)
 {
-  GtkWidget    *area = GTK_BIN (window)->child; 
+  GtkWidget    *area = GTK_BIN (window)->child;
 
   g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
@@ -1788,57 +1885,85 @@ hd_home_window_show_information_banner (HDHomeWindow *window,
 }
 #endif
 
-
-#if 0
 static void
-scale_changed (GtkRange *range, GtkWidget *area)
+hd_home_window_cancel_note_hide (HDHomeWindow *window)
 {
-  gint  alpha;
-  
-  alpha = (gint) gtk_range_get_value (range);
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
 
-  g_debug ("value_changed");
+  if (priv->cancel_note_update_timeout > 0)
+    {
+      g_source_remove (priv->cancel_note_update_timeout);
+      priv->cancel_note_update_timeout = 0;
+    }
 
-  g_object_set (area,
-                "applet-default-alpha", alpha,
-                NULL);
+  if (GTK_IS_WIDGET (priv->cancel_note))
+    {
+      gtk_widget_destroy (priv->cancel_note);
+      priv->cancel_note = NULL;
+    }
 }
 
 static void
-hd_home_window_adjust_alpha_activate (HDHomeWindow *window)
+hd_home_window_cancel_note_response (HDHomeWindow *window)
 {
-  GtkWidget    *area, *dialog, *scale;
-  gint          alpha;
-  gint          response;
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
 
-  area = hildon_home_window_get_area (HILDON_HOME_WINDOW (window));
+  hd_home_window_cancel_note_hide (window);
 
-  g_object_get (area,
-                "applet-default-alpha", &alpha,
-                NULL);
-
-  dialog = gtk_dialog_new_with_buttons ("Adjust applets' transparency",
-                                        GTK_WINDOW (window),
-                                        GTK_DIALOG_MODAL | 
-                                          GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-                                        NULL);
-  gtk_widget_set_size_request (dialog, 300, -1);
-
-  scale = gtk_hscale_new_with_range (0.0,
-                                     100.0,
-                                     1.0);
-  gtk_widget_show (scale);
-  g_signal_connect (scale, "value-changed",
-                    G_CALLBACK (scale_changed),
-                    area);
-  gtk_range_set_value (GTK_RANGE (scale), (gdouble)alpha);
-  
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), scale);
-
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
-  
-  gtk_widget_destroy (dialog);
+  priv->background_cancelled = TRUE;
 }
 
-#endif
+static gboolean
+hd_home_window_cancel_note_update (GtkWidget *prog_bar)
+{
+  if (!GTK_IS_PROGRESS_BAR (prog_bar))
+    return FALSE;
+
+  gtk_progress_bar_pulse (GTK_PROGRESS_BAR (prog_bar));
+  return TRUE;
+}
+
+static void
+hd_home_window_cancel_note_show (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget            *label, *prog_bar;
+
+  if (priv->cancel_note)
+    return;
+
+  priv->cancel_note =
+      gtk_dialog_new_with_buttons ("",
+                                   NULL,
+                                   GTK_DIALOG_MODAL |
+                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   HH_SET_BG_CANCEL, GTK_RESPONSE_CANCEL,
+                                   NULL);
+
+  label = gtk_label_new (HH_CANCEL_BG_TEXT);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (priv->cancel_note)->vbox),
+                     label);
+
+  prog_bar = gtk_progress_bar_new ();
+  gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (prog_bar), 0.2);
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (prog_bar), 0.2);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (priv->cancel_note)->vbox),
+                     prog_bar);
+
+  priv->cancel_note_update_timeout =
+      g_timeout_add (500,
+                     (GSourceFunc)hd_home_window_cancel_note_update,
+                     prog_bar);
+
+  gtk_dialog_set_has_separator (GTK_DIALOG (priv->cancel_note), FALSE);
+
+  g_signal_connect_swapped (priv->cancel_note, "response",
+                            G_CALLBACK (hd_home_window_cancel_note_response),
+                            window);
+
+  gtk_widget_realize (priv->cancel_note);
+  gdk_window_set_decorations (priv->cancel_note->window,
+                              GDK_DECOR_BORDER);
+
+  gtk_widget_show_all (priv->cancel_note);
+}
