@@ -623,10 +623,6 @@ hd_switcher_menu_constructor (GType gtype,
                           G_CALLBACK (hd_switcher_menu_item_activated),
                           (gpointer)switcher); 
 
-  hildon_desktop_popup_menu_add_item
-   (switcher->priv->menu_applications, 
-    GTK_MENU_ITEM (gtk_separator_menu_item_new ()));
-
   hd_switcher_menu_populate_notifications (switcher);
 
   hd_switcher_menu_check_content (switcher);
@@ -1078,7 +1074,7 @@ static void
 hd_switcher_menu_create_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
 {
   GList *children = NULL, *apps = NULL, *l;
-  GtkWidget *separator;
+  GtkWidget *separator = NULL;
 
   children =
     hildon_desktop_popup_menu_get_children (switcher->priv->menu_applications);
@@ -1108,16 +1104,16 @@ hd_switcher_menu_create_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
     const GList * children = hd_entry_info_get_children(l->data);
     const GList * child;
 
+    if (l == apps)
+    {
+      separator = gtk_separator_menu_item_new ();
+
+      hildon_desktop_popup_menu_add_item
+       (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
+    }
+
     for (child = children; child != NULL; child = child->next)
     {
-      if (l == apps && child == children)
-      {
-        separator = gtk_separator_menu_item_new ();
-
-        hildon_desktop_popup_menu_add_item
-         (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
-      }	      
-	    
       HDEntryInfo *entry = child->data;
 
       menu_item = hd_switcher_menu_item_new_from_entry_info (entry, TRUE);
@@ -1137,11 +1133,14 @@ hd_switcher_menu_create_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
       switcher->priv->active_menu_item = menu_item;
     }
 
-    /* append the separator for this app*/
-    separator = gtk_separator_menu_item_new ();
+    /* Append the separator for this app*/
+    if (l->next != NULL)
+    {
+      separator = gtk_separator_menu_item_new ();
 
-    hildon_desktop_popup_menu_add_item
-      (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
+      hildon_desktop_popup_menu_add_item
+        (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
+    }
   }
 
   g_list_free (apps);
@@ -1174,6 +1173,8 @@ hd_switcher_menu_remove_info_cb (HDWM *hdwm,
 
   for (l = children; l != NULL; l = g_list_next (l))
   {
+    GtkMenuItem *separator = NULL;
+	  
     if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))
     {	    
       _info = hd_switcher_menu_item_get_entry_info (HD_SWITCHER_MENU_ITEM (l->data));
@@ -1183,9 +1184,20 @@ hd_switcher_menu_remove_info_cb (HDWM *hdwm,
         hildon_desktop_popup_menu_remove_item (switcher->priv->menu_applications,
 	  				       GTK_MENU_ITEM (l->data));
 
-        if (l->prev && GTK_IS_SEPARATOR_MENU_ITEM (l->prev->data))
+        if (l == children)
+	{
+          if (l->next && GTK_IS_SEPARATOR_MENU_ITEM (l->next->data))
+	    separator = GTK_MENU_ITEM (l->next->data);
+	}
+	else
+        {
+	  if (l->prev && GTK_IS_SEPARATOR_MENU_ITEM (l->prev->data))
+	    separator = GTK_MENU_ITEM (l->prev->data);
+	}
+
+	if (separator != NULL)
           hildon_desktop_popup_menu_remove_item (switcher->priv->menu_applications,
-                                                 GTK_MENU_ITEM (l->prev->data));		
+	  				         separator);
 
 	if (info == switcher->priv->last_urgent_info)
         {
@@ -1198,7 +1210,9 @@ hd_switcher_menu_remove_info_cb (HDWM *hdwm,
     }
   }
 
-  g_list_free (children);  
+  g_list_free (children);
+
+  hd_switcher_menu_changed_stack_cb (switcher->hdwm, NULL, switcher);  
 
   hd_switcher_menu_check_content (switcher);
 }
@@ -1490,6 +1504,7 @@ hd_switcher_menu_notification_changed_cb (GtkTreeModel   *tree_model,
   gchar *summary = NULL, *body = NULL;
   const gchar *category = NULL;
   GHashTable *hints;
+  GList *children;
   GValue *hint;
   guint id;
   GtkWidget *menu_item;
@@ -1516,14 +1531,22 @@ hd_switcher_menu_notification_changed_cb (GtkTreeModel   *tree_model,
   {	  
     category = g_value_get_string (hint);
 
-    if (g_str_equal (category, "system.note.dialog"))
+    if (g_str_has_prefix (category, "system.note"))
       goto out;	  
   }
   
   if (switcher->priv->last_iter_added == NULL)
     hd_switcher_menu_notification_deleted_cb 
       (HILDON_DESKTOP_NOTIFICATION_MANAGER (tree_model), id, switcher);	    
+
+  children = 
+    hildon_desktop_popup_menu_get_children (switcher->priv->menu_notifications);
   
+  if (g_list_length (children) > 0)
+    hildon_desktop_popup_menu_add_item
+      (switcher->priv->menu_notifications, 
+       GTK_MENU_ITEM (gtk_separator_menu_item_new ()));
+
   menu_item = 
     hd_switcher_menu_item_new_from_notification 
       (id, icon, summary, body, TRUE);
@@ -1534,11 +1557,9 @@ hd_switcher_menu_notification_changed_cb (GtkTreeModel   *tree_model,
     (switcher->priv->menu_notifications, GTK_MENU_ITEM (menu_item));
 
   hd_switcher_menu_replace_blinking_icon (switcher, icon);
+
   switcher->priv->last_urgent_info = NULL;
 
-  hildon_desktop_popup_menu_add_item
-    (switcher->priv->menu_notifications, 
-     GTK_MENU_ITEM (gtk_separator_menu_item_new ()));	 
 out:    	    
   switcher->priv->last_iter_added = NULL;
 
@@ -1562,6 +1583,7 @@ hd_switcher_menu_notification_deleted_cb (HildonDesktopNotificationManager   *nm
   {
     if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))	  
     {
+      GtkMenuItem *separator = NULL;
       gint _id =
         hd_switcher_menu_item_get_notification_id (HD_SWITCHER_MENU_ITEM (l->data));	    
 	  
@@ -1570,9 +1592,21 @@ hd_switcher_menu_notification_deleted_cb (HildonDesktopNotificationManager   *nm
         hildon_desktop_popup_menu_remove_item (switcher->priv->menu_notifications,
 	                                       GTK_MENU_ITEM (l->data));
 
-        if (l->prev && GTK_IS_SEPARATOR_MENU_ITEM (l->prev->data))
+        if (l == children)
+	{
+          if (l->next && GTK_IS_SEPARATOR_MENU_ITEM (l->next->data))
+	    separator = GTK_MENU_ITEM (l->next->data);
+	}
+	else
+        {
+	  if (l->prev && GTK_IS_SEPARATOR_MENU_ITEM (l->prev->data))
+	    separator = GTK_MENU_ITEM (l->prev->data);
+	}
+
+	if (separator != NULL)
           hildon_desktop_popup_menu_remove_item (switcher->priv->menu_notifications,
-	  				         GTK_MENU_ITEM (l->prev->data));		
+	  				         separator);
+
         break;
       }	    
     }
@@ -1638,4 +1672,3 @@ hd_switcher_menu_populate_notifications (HDSwitcherMenu *switcher)
     while (gtk_tree_model_iter_next (nm, &iter));
   }	  
 }
-
