@@ -136,6 +136,8 @@ struct _HNAppButtonPrivate
   guint is_blinking  : 1;
   guint is_thumbable : 1;
 
+  HDEntryInfo *last_entry;
+
   HNAppSwitcher *app_switcher;
 };
 
@@ -801,6 +803,7 @@ hn_app_button_press_event (GtkWidget      *widget,
   GtkButton *button;
   GtkToggleButton *toggle_button;
   GtkToggleButton *tmp_button = NULL;
+  GtkWidget *toplevel_window;
   GSList *l;
   HDEntryInfo *info;
   
@@ -815,8 +818,10 @@ hn_app_button_press_event (GtkWidget      *widget,
   /* remember which button was used to press this button */
   HN_DBG("App button pressed using button %d", event->button);
 
+  toplevel_window = gtk_widget_get_toplevel (widget);
+
   hd_wm_activate_window (HD_TN_DEACTIVATE_KEY_FOCUS,
-		         gtk_widget_get_parent_window (widget));
+		         toplevel_window->window);
 
   if (event->button == APP_BUTTON_THUMBABLE)
     priv->is_thumbable = TRUE;
@@ -997,6 +1002,8 @@ hn_app_button_init (HNAppButton *app_button)
   priv->is_blinking = FALSE;
   priv->is_thumbable = FALSE;
 
+  priv->last_entry = NULL;
+
   gtk_widget_set_size_request (GTK_WIDGET (app_button), -1, BUTTON_HEIGHT);
   gtk_widget_set_sensitive (GTK_WIDGET (app_button), FALSE);
 }
@@ -1094,6 +1101,14 @@ hn_app_button_get_entry_info (HNAppButton *button)
   return button->priv->info;
 }
 
+HDEntryInfo *
+hn_app_button_get_last_entry_info (HNAppButton *button)
+{
+  g_return_val_if_fail (HN_IS_APP_BUTTON (button), NULL);
+
+  return button->priv->last_entry;
+}
+
 static GdkPixbuf *
 get_pixbuf_for_entry_info (HDEntryInfo *info)
 {
@@ -1217,65 +1232,68 @@ hn_app_button_set_entry_info (HNAppButton *button,
 {
   g_return_if_fail (HN_IS_APP_BUTTON (button));
 
+  button->priv->last_entry = button->priv->info;
   button->priv->info = NULL;
 
   HN_MARK();
 
   if (info)
-    {
-      GdkPixbuf *app_pixbuf;
+  {
+    GdkPixbuf *app_pixbuf;
       
-      app_pixbuf = get_pixbuf_for_entry_info (info);
-      if (app_pixbuf)
-	{
-          GdkPixbuf *pixbuf;
+    app_pixbuf = get_pixbuf_for_entry_info (info);
 
-	  /* compose the application icon with the number of
-	   * instances running
-	   */
-	  pixbuf = hn_app_button_compose_app_pixbuf (button, app_pixbuf, info);
-	  if (pixbuf)
-            {
-              gtk_image_set_from_pixbuf (GTK_IMAGE (button->priv->icon),
-			                 pixbuf);
-	      g_object_unref (pixbuf);
-	    }
-	  else
-            gtk_image_set_from_pixbuf (GTK_IMAGE (button->priv->icon),
-			               app_pixbuf);
+    if (app_pixbuf)
+    {
+      GdkPixbuf *pixbuf;
 
-	  g_object_unref (app_pixbuf);
-	}
+      /* compose the application icon with the number of
+       * instances running
+       */
+      pixbuf = hn_app_button_compose_app_pixbuf (button, app_pixbuf, info);
+
+      if (pixbuf)
+      {
+        gtk_image_set_from_pixbuf (GTK_IMAGE (button->priv->icon),
+ 		                   pixbuf);
+        g_object_unref (pixbuf);
+      }
       else
-	HN_DBG ("Unable to find the icon (even the default one)");
+        gtk_image_set_from_pixbuf (GTK_IMAGE (button->priv->icon),
+    		                   app_pixbuf);
+
+      g_object_unref (app_pixbuf);
+    }
+    else
+      g_debug ("Unable to find the icon (even the default one)");
 
       /* the newly composed image is static */
-      if (button->priv->is_blinking &&
-	  button->priv->app_switcher && 
-	  !hn_app_switcher_get_system_inactivity (button->priv->app_switcher))
-      {
-        hn_app_button_icon_animation (button->priv->icon, button->priv->is_blinking);
-      }
-      
-      gtk_widget_show (button->priv->icon);
-      gtk_widget_set_sensitive (GTK_WIDGET (button), TRUE);
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
-		                    hd_entry_info_is_active (info));
-      
-      g_object_set (G_OBJECT (button), "can-focus", TRUE, NULL);
-      
-      button->priv->info = info;
-    }
-  else
+    if (button->priv->is_blinking &&
+        button->priv->app_switcher && 
+        !hn_app_switcher_get_system_inactivity (button->priv->app_switcher))
     {
-      gtk_widget_hide (button->priv->icon);
-      gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
-
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
-      gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (button));
-
-      g_object_set (G_OBJECT (button), "can-focus", FALSE, NULL);
+      hn_app_button_icon_animation (button->priv->icon, button->priv->is_blinking);
     }
+      
+    gtk_widget_show (button->priv->icon);
+    gtk_widget_set_sensitive (GTK_WIDGET (button), TRUE);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
+	                          hd_entry_info_is_active (info));
+      
+    g_object_set (G_OBJECT (button), "can-focus", TRUE, NULL);
+      
+    button->priv->info = info;
+  }
+  else
+  {
+    gtk_widget_hide (button->priv->icon);
+    gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+    gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (button));
+
+    g_object_set (G_OBJECT (button), "can-focus", FALSE, NULL);
+  }
 }
 
 void
